%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% WN_CONNECT source v0.00 : wn_sim_adjectives module
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
/**********************************************************
This module defines predicates that find the adjectives which are similar
in meaning to a input adjective. Do not confuse "similar" with "synonym".
Synonym words are grouped in a synset and they are words equals in meaning.
In other words, synonym words must have the maximum (top) degree of similarity.

Most of predicates defined in this module a based on the operator "sim"

-------------------------
sim(synset_id,synset_id).
-------------------------
The "sim" operator specifies that the second synset is similar in meaning
to the first synset. This means that the second synset is a satellite of
the first synset, which is the cluster head. This relation only holds for
adjective synsets contained in adjective clusters.

The two addressed synsets are either two head synsets, or one head synset
and one satellite synset. There is no matching sim clause for two satellite
synsets. Because, if they would have similar meanings, they would be grouped
together in one synset.

Therefore, the predicates defined in this module are for adjectives and do not
work for other parts of speech.
**********************************************************/


:- module(wn_similar_adjectives, [
wn_sim_adjectives_of/2,
wn_display_sim_adjectives_of/1 %
]).

:- use_module(wn_synsets).



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% wn_sim_adjectives_of(Adjective, List_sim_SynSets) is true if List_sim_SynSets
%% is a list of similar adjectives synsets of the adjective Adjective.
%% Adjective is a word.
%% Only adjectives can be similar one of each other. That is, words of type "a" or "s".
%% There is no matching sim clause for two satellite synsets. Because, if they would
%% have similar meanings, they would be grouped together in one synset.
%%
%%% The word "Adjective" is a term with the following syntax:
%%%                      Word[:SS_type[:Sense_num]]
%%% Where ss_type is a one character code indicating the synset type:
%%    n NOUN
%%    v VERB
%%    a ADJECTIVE
%%    s ADJECTIVE SATELLITE
%%    r ADVERB
%%
%%% and "Sense_num" specifies the sense number of the word, within the part of
%% speech encoded in the synset_id. "Sense_num" is a natural number: 1, 2, 3, ...
%%%%%%%%%%

wn_sim_adjectives_of(Adjective:SS_type:SS_num,List_sim_SynSets) :-
    !,
    wordnet:wn_s(A_Synset_ID, _W_num, Adjective, SS_type, SS_num, _W_Tag_count),
    ((SS_type=a ; SS_type=s) ->
        findall(Synset_ID, wordnet:wn_sim(A_Synset_ID, Synset_ID), List_sim_SynSets)
        ;
        write(">>>> "), write(Adjective:SS_type), nl,
        wordnet:wn_g(A_Synset_ID, Glos),
        write(Glos), nl,
        write("Only adjectives can be related by the 'sim' relation"),
        nl,
        fail
    ).


wn_sim_adjectives_of(Adjective:SS_type,List_sim_SynSets) :-
    !,
    wordnet:wn_s(A_Synset_ID, _W_num, Adjective, SS_type, _SS_num, _W_Tag_count),
    ((SS_type=a ; SS_type=s) ->
        findall(Synset_ID, wordnet:wn_sim(A_Synset_ID, Synset_ID), List_sim_SynSets)
        ;
        write(">>>> "), write(Adjective:SS_type), nl,
        wordnet:wn_g(A_Synset_ID, Glos),
        write(Glos), nl,
        write("Only adjectives can be related by the 'sim' relation"),
        nl,
        fail
    ).


wn_sim_adjectives_of(Adjective,List_sim_SynSets) :-
        wordnet:wn_s(A_Synset_ID, _W_num, Adjective, SS_type, _SS_num, _W_Tag_count),
        ((SS_type=a ; SS_type=s) ->
            findall(Synset_ID, wordnet:wn_sim(A_Synset_ID, Synset_ID), List_sim_SynSets)
            ;
            write(">>>> "), write(Adjective:SS_type), nl,
            wordnet:wn_g(A_Synset_ID, Glos),
            write(Glos), nl,
            write("Only adjectives can be related by the 'sim' relation"),
            nl,
            fail
        ).



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% wn_display_sim_adjectives_of(+Adjective)
%%% Given a word (term) "Adjective", prints the list of its
%%% similar synsets
%%%
%%% The word "Adjective" is a term with the following syntax:
%%%                      Word[:SS_type[:Sense_num]]
%%% as explained in the predicate wn_sim_adjectives_of/2

wn_display_sim_adjectives_of(Adjective:SS_type:SS_num) :-
    !,
    wordnet:wn_s(_Synset_ID, _W_num, Adjective, SS_type, SS_num, _W_Tag_count),
    write(">>> SIMILAR ADJECTIVES OF "), write(Adjective:SS_type:SS_num), write(' :'), nl,
    wn_sim_adjectives_of(Adjective:SS_type:SS_num,List_sim_SynSets),
    display_sim_adjectives_list(List_sim_SynSets).

wn_display_sim_adjectives_of(Adjective:SS_type) :-
    !,
    wordnet:wn_s(_Synset_ID, _W_num, Adjective, SS_type, SS_num, _W_Tag_count),
    write(">>> SIMILAR ADJECTIVES OF "), write(Adjective:SS_type:SS_num), write(' :'), nl,
    wn_sim_adjectives_of(Adjective:SS_type,List_sim_SynSets),
    display_sim_adjectives_list(List_sim_SynSets).


wn_display_sim_adjectives_of(Adjective) :-
    wordnet:wn_s(_Synset_ID, _W_num, Adjective, SS_type, SS_num, _W_Tag_count),
    write(">>> SIMILAR ADJECTIVES OF "), write(Adjective:SS_type:SS_num), write(' :'), nl,
    wn_sim_adjectives_of(Adjective,List_sim_SynSets),
    display_sim_adjectives_list(List_sim_SynSets).


/********* INITIAL DEFINITION FOR A DIFFERENT IDEA
wn_display_sim_adjectives_of(Adjective) :-
    wordnet:wn_s(A_Synset_ID, _W_num, Adjective, SS_type, SS_num, _W_Tag_count),
    wn_sim_adjectives_of(Adjective,List_sim_SynSets),
    display_sim_adjectives_list(List_sim_SynSets).
***************************************************************/
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% display_sim_adjectives_list(List_sim_SynSets)

display_sim_adjectives_list([]).
display_sim_adjectives_list([SynSet_ID|List_SynSet_IDs]):-
        wn_synsets:wn_synset_components(SynSet_ID, Synset_Words, verbose(yes)),
        write(' ~~ '),
        write(Synset_Words), nl,
        display_sim_adjectives_list(List_SynSet_IDs).











